@@ -55,16 +55,16 @@ public:
 		child = myChild;
 		isComplete = myIsComplete;
 	}
-};
-
-class GymReturn
-{
-public:
-	double reward;
-	bool done;
-	vector<int> state;
-	string info;
-	GymReturn() {}
+	Entity(const Entity& origEntity) {
+		assetType = origEntity.assetType;
+		timeToComplete = origEntity.timeToComplete;
+		timeQueueFree = origEntity.timeQueueFree;
+		timeChronoLeft = origEntity.timeChronoLeft;
+		energy = origEntity.energy;
+		executor = origEntity.executor;
+		child = origEntity.child;
+		isComplete = origEntity.isComplete;
+	}
 };
 
 class Names
@@ -157,8 +157,8 @@ public:
 		colossus = 11;
 		disruptor = 12;
 		phoenix = 13;
-		voidray = 14;
-		oracle = 15;
+		oracle = 14;
+		voidray = 15;
 		tempest = 16;
 		carrier = 17;
 		warpzealot = 18;
@@ -222,7 +222,6 @@ class ProtossMacroEnv
 public:
 	double minerals;
 	double gas;
-	double reward;
 	int mProbeCount;
 	int gProbeCount;
 	int maxSupply;
@@ -232,26 +231,399 @@ public:
 	double REGEN_RATE;
 	vector<AssetStats> assetList;
 	vector<Entity> entityList;
-	int inProgressEntities[100];
-	int inProgressCount;
 	vector<int> goalState;
 	vector<int> state;
-	GymReturn gymReturn;
 	bool done;
 	Names n;
 
-	ProtossMacroEnv() {
+	ProtossMacroEnv(const Names& myNames, const vector<AssetStats>& myAssetList) {
 		REGEN_RATE = 1;
-		n = Names();
-		gymReturn = GymReturn();
-		loadStatCatalog(n, state, assetList);
+		n = myNames;
+		assetList = myAssetList;
+		for (int i = 0; i < assetList.size(); i++) {
+			state.push_back(0);
+		}
 		initEntityList();
 		initGlobalState();
 		initGoalState();
-		inProgressCount = 0;
 	}
 
-	void loadStatCatalog(Names& n, vector<int>& state, vector<AssetStats>& assetList) {
+	ProtossMacroEnv(const ProtossMacroEnv& env) {
+		//copy raw 
+		minerals = env.minerals;
+		gas = env.gas;
+		mProbeCount = env.mProbeCount;
+		gProbeCount = env.gProbeCount;
+		maxSupply = env.maxSupply;
+		maxFutureSupply = env.maxFutureSupply;
+		supply = env.supply;
+		time = env.time;
+		REGEN_RATE = env.REGEN_RATE;
+
+		//shallow copy
+		n = env.n;
+		assetList = env.assetList;
+
+		//deep copy
+		copyEntityList(env.entityList);
+		for (int i : env.state) {
+			state.push_back(i);
+		}
+	}
+
+	void initGoalState() {
+		for (size_t i = 0; i < assetList.size(); i++) {
+			goalState.push_back(0);
+		}
+		goalState[n.probe] = 14;
+		goalState[n.zealot] = 1;
+		goalState[n.sentry] = 3;
+	}
+	void initGlobalState() {
+		minerals = 50;
+		gas = 0;
+		mProbeCount = 0;
+		gProbeCount = 0;
+		time = 0;
+
+		//supply related items
+		supply = 0;
+		maxSupply = 0;
+		for (Entity const& e : entityList) {
+			if (e.assetType == n.probe) {
+				mProbeCount++;
+			}
+			maxSupply += assetList[e.assetType].supProvide;
+			supply += assetList[e.assetType].supCost;
+		}
+		maxFutureSupply = maxSupply;
+	}
+	void initEntityList() {
+		entityList.push_back(loadEntity(n.dummy, 0, 0, true));
+		entityList.push_back(loadEntity(n.nexus, 0, 0, true));
+		entityList.back().energy = 50;
+		state[n.nexus]++;
+		for (int i = 0; i < 12; i++) {
+			entityList.push_back(loadEntity(n.probe, 0, 0, true));
+			state[n.probe]++;
+		}
+	}
+	void copyEntityList(const vector<Entity>& origList) {
+		for (const Entity& e : origList) {
+			entityList.push_back(Entity(e));
+		}
+	}
+	void reset() {
+		entityList.clear();
+		time = 0;
+		for (int i = 0; i < state.size(); i++) {
+			state[i] = 0;
+		}
+		initEntityList();
+		initGlobalState();
+	}
+	Entity loadEntity(int action, int executor, int target, bool complete) {
+		double timeToComplete;
+		if (complete) {
+			timeToComplete = 0;
+		}
+		else {
+			double timeReduce = 0;
+			if (executor > 0 && entityList[executor].timeChronoLeft > 0) {
+				timeReduce = min(10, assetList[action].duration / 3);
+			}
+			timeToComplete = assetList[action].duration + time - timeReduce;
+		}
+		return Entity(action, timeToComplete, time, 0, 0, executor, target, complete);
+	}
+	void step(int action, int executor, int chronoTarget) {
+		double timeToAction = computeTimeToAction(action, executor, chronoTarget);
+		double slidingTime = timeToAction;
+		//need to iterate through all incomplete items
+		while (true) {
+			int idx = getNextComplete();
+			if (idx == -1) break;
+			double timeComplete = entityList[idx].timeToComplete - time;
+			if (timeComplete <= slidingTime) {
+				elapseTime(timeComplete);
+				completeEvent(idx);
+				slidingTime = computeTimeToAction(action, executor, chronoTarget);
+			}
+			else break;
+		}
+		elapseTime(slidingTime);
+		completeAction(action, executor, chronoTarget);
+		//done = isDone();
+	}
+
+	double getTimeNextSupplyDone() {
+		double minTime = 10000;
+		for (const Entity& e : entityList) {
+			if (!e.isComplete && assetList[e.assetType].supProvide > 0) {
+				minTime = min(e.timeToComplete, minTime);
+			}
+		}
+		return minTime - time;
+	}
+
+	double computeTimeToAction(int action, int executor, int chronoTarget) {
+		if (action == n.chronoboost) {
+			double energyAvailableAt;
+			if (entityList[executor].isComplete) {
+				energyAvailableAt = max(0.0, 50 - entityList[executor].energy) / REGEN_RATE;
+			}
+			else {
+				energyAvailableAt = entityList[executor].timeToComplete - time;
+			}
+			return max(entityList[chronoTarget].timeChronoLeft, energyAvailableAt);
+		}
+
+		double timeMinerals;
+		double timeGas;
+		double timeSupply;
+		double timeQueue;
+		const AssetStats& a = assetList[action];
+		double mineralRate = mRate();
+		double gasRate = gRate();
+
+		//also need to consider when executor has a free queue
+
+		if (mineralRate == 0) timeMinerals = 0;
+		else {
+			timeMinerals = (a.minCost - minerals) / mineralRate;
+		}
+		if (gasRate == 0) timeGas = 0;
+		else {
+			timeGas = (a.gasCost - gas) / gasRate;
+		}
+		if (maxSupply >= supply + a.supCost) timeSupply = 0;
+		else {
+			timeSupply = getTimeNextSupplyDone();
+		}
+		if (executor > 0) {
+			timeQueue = entityList[executor].timeQueueFree - time;
+		}
+		else {
+			timeQueue = 0;
+
+		}
+		double largest = timeMinerals;
+		if (timeGas > largest) largest = timeGas;
+		if (timeSupply > largest) largest = timeSupply;
+		if (timeQueue > largest) largest = timeQueue;
+		return largest;
+	}
+
+	double mRate() {
+		return mProbeCount;
+	}
+	double gRate() {
+		return gProbeCount;
+	}
+
+	void elapseTime(double timeElapse) {
+		minerals += timeElapse * mRate();
+		gas += timeElapse * gRate();
+		for (Entity& e : entityList) {
+			//e.timeChronoLeft = max(0.0, e.timeChronoLeft - timeElapse);
+			//e.timeQueueFree = max(0.0, e.timeQueueFree - timeElapse);
+			//e.timeToComplete = max(0.0, e.timeToComplete - timeElapse);
+			e.energy = min(200.0, e.energy + timeElapse * REGEN_RATE);
+		}
+		time += timeElapse;
+	}
+
+	void completeEvent(int idx) {
+		int aType = entityList[idx].assetType;
+		if (aType == n.nexus) {
+			entityList[idx].energy = 50;
+		}
+		else if (aType == n.probe) {
+			mProbeCount++;
+		}
+		else if (aType == n.assimilator) {
+			int probesChange = max(0, min(3, mProbeCount - 3));
+			mProbeCount -= probesChange;
+			gProbeCount += probesChange;
+		}
+		maxSupply += assetList[entityList[idx].assetType].supProvide;
+		entityList[idx].isComplete = true;
+	}
+
+	void completeAction(int action, int executor, int target) {
+		if (action == n.chronoboost) {
+			entityList[executor].energy -= 50;
+			entityList[target].timeChronoLeft = 20;
+			double timeStillQueued = max(0.0, entityList[target].timeQueueFree - time);
+			double timeShift = min(10.0, timeStillQueued / 3);
+			entityList[target].timeQueueFree -= timeShift;
+			if (entityList[target].child > 0) {
+				//production structure is currently building a unit
+				entityList[entityList[target].child].timeToComplete -= timeShift;
+			}
+			return;
+		}
+		const AssetStats& a = assetList[action];
+		maxFutureSupply += a.supProvide;
+		minerals -= a.minCost;
+		gas -= a.gasCost;
+		supply += a.supCost;
+		entityList.push_back(loadEntity(action, executor, target, false));
+		if (executor > 0) {
+			entityList[executor].timeQueueFree = a.duration + time;
+			entityList[executor].child = entityList.size() - 1;
+		}
+		state[action] += 1;
+	}
+
+	int getNextComplete() {
+		double minCompleteTime = 10000;
+		int nextComplete = -1;
+		for (size_t i = 0; i < entityList.size(); i++) {
+			const Entity& e = entityList[i];
+			if (!e.isComplete && e.timeToComplete < minCompleteTime) {
+				minCompleteTime = e.timeToComplete;
+				nextComplete = i;
+			}
+		}
+
+		return nextComplete;
+	}
+
+	bool isDone() {
+		//check if entity counter >= goalstate
+		for (size_t i = 0; i < state.size(); i++) {
+			if (state[i] < goalState[i]) {
+				return false;
+			}
+		}
+		return true;
+	}
+
+	bool techExists(int i) {
+		for (int req : assetList[i].techReqs) {
+			if (state[req] == 0) {
+				return false;
+			}
+		}
+		return true;
+	}
+
+	bool maxAssimilators(const AssetStats& a) {
+		return (a.name == n.assimilator && state[n.assimilator] > state[n.nexus] * 2);
+	}
+
+	vector<int > getViableActions() {
+		//needs to pass tech, supply, resource, maxCount checks
+		//vector<int > viableActions;
+		vector<int > viableActions(100);
+		int viableCount = 0;
+		for (const AssetStats& a : assetList) {
+			if (maxAssimilators(a)) continue;
+			bool techExists = true;
+			for (int req : a.techReqs) {
+				if (state[req] == 0) {
+					techExists = false; break;
+				}
+			}
+			bool isViable = (
+				techExists
+				&& (a.supCost + supply <= maxFutureSupply)
+				&& (mProbeCount > 0 || a.minCost == 0)
+				&& (gProbeCount > 0 || a.gasCost == 0)
+				&& (state[a.name] < a.maxQuantity)
+				);
+			if (isViable) {
+				viableActions[viableCount++] = a.name;
+			}
+		}
+		//chronoboost
+		if (state[n.nexus] > 0) {
+			viableActions[viableCount++] = n.chronoboost;
+		}
+		viableActions.resize(viableCount);
+		return viableActions;
+	}
+	vector<int> getViableExecutors(int action) {
+		int executorName;
+		if (action == n.chronoboost) {
+			executorName = n.nexus;
+		}
+		else { executorName = assetList[action].parent; }
+		vector<int> viableExecutors;
+		if (executorName == n.dummy) return viableExecutors;
+		for (int i = 0; i < entityList.size(); i++) {
+			if (entityList[i].assetType == executorName) {
+				viableExecutors.push_back(i);
+			}
+		}
+		return viableExecutors;
+	}
+
+	vector<int> getViableTargets(int action, int executor) {
+		vector<int> viableTargets;
+		//only chronoboost has targets
+		if (action != n.chronoboost) return viableTargets;
+		for (int i = 0; i < entityList.size(); i++) {
+			if (assetList[entityList[i].assetType].canChrono) {
+				viableTargets.push_back(i);
+			}
+		}
+		return viableTargets;
+	}
+
+	vector<vector<double>> getState() {
+		int maxEntities = 100;
+		vector<vector<double>> returnState(maxEntities, vector<double>(6, 0));
+		for (int i = 0; i < entityList.size(); i++) {
+			if (i == maxEntities)break;
+			returnState[i][0] = entityList[i].assetType;
+			returnState[i][1] = max(0.0, entityList[i].timeToComplete - time);
+			returnState[i][2] = max(0.0, entityList[i].timeQueueFree - time);
+			returnState[i][3] = max(0.0, entityList[i].timeChronoLeft - time);
+			returnState[i][4] = entityList[i].energy;
+			returnState[i][5] = (double)entityList[i].isComplete;
+		}
+		return returnState;
+	}
+
+	int pickNextExecutor(vector<int> viableExecutors) {
+		if (viableExecutors.size() == 0) return -1;
+		double timeFree = 10000;
+		int nextExecutor = 0;
+		for (int x : viableExecutors) {
+			if (entityList[x].timeQueueFree < timeFree) {
+				timeFree = entityList[x].timeQueueFree;
+				nextExecutor = x;
+			}
+		}
+		return nextExecutor;
+	}
+};
+
+class GeneralLoader
+{
+public:
+	vector<AssetStats> assetList;
+	Names n;
+
+	GeneralLoader() {
+		n = Names();
+		loadStatCatalog();
+	}
+
+	//create an environment from here that will have references to this object
+	ProtossMacroEnv createFreshEnv() {
+		ProtossMacroEnv env = ProtossMacroEnv(n, assetList);
+		return env;
+	}
+
+	ProtossMacroEnv deepCopyEnv(ProtossMacroEnv& origEnv) {
+		ProtossMacroEnv env = ProtossMacroEnv(origEnv);
+		return env;
+	}
+
+	void loadStatCatalog() {
 		assetList = {
 			//placeholder
 			AssetStats(n.dummy, 0, 0, 0, 0, 0, n.dummy, { n.dummy }, 1, false),
@@ -325,346 +697,6 @@ public:
 			AssetStats(n.warpgateresearch, 50, 50, 0, 0,100,n.cyberneticscore,{n.cyberneticscore}, 1, false),
 			AssetStats(n.gravitoncatapult, 150, 150, 0,0, 57,n.fleetbeacon,{n.fleetbeacon}, 1, false),
 		};
-		for (size_t i = 0; i < assetList.size(); i++) {
-			state.push_back(0);
-		}
-	}
-	void initGoalState() {
-		for (size_t i = 0; i < assetList.size(); i++) {
-			goalState.push_back(0);
-		}
-		goalState[n.probe] = 14;
-		goalState[n.zealot] = 1;
-		goalState[n.sentry] = 3;
-	}
-	void initGlobalState() {
-		minerals = 50;
-		gas = 0;
-		mProbeCount = 0;
-		maxSupply = 0;
-		gProbeCount = 0;
-		time = 0;
-		supply = 0;
-		for (Entity const& e : entityList) {
-			if (e.assetType == n.probe) {
-				mProbeCount++;
-			}
-			maxSupply += assetList[e.assetType].supProvide;
-			supply += assetList[e.assetType].supCost;
-		}
-		maxFutureSupply = maxSupply;
-	}
-	void initEntityList() {
-		entityList.push_back(loadEntity(n.dummy, 0, 0, true));
-		entityList.push_back(loadEntity(n.nexus, 0, 0, true));
-		entityList.back().energy = 50;
-		state[n.nexus]++;
-		for (int i = 0; i < 12; i++) {
-			entityList.push_back(loadEntity(n.probe, 0, 0, true));
-			state[n.probe]++;
-		}
-	}
-	void reset() {
-		entityList.clear();
-		for (int i = 0; i < state.size(); i++) {
-			state[i] = 0;
-		}
-		initEntityList();
-		initGlobalState();
-	}
-	Entity loadEntity(int action, int executor, int target, bool complete) {
-		double timeToComplete;
-		if (complete) {
-			timeToComplete = 0;
-		}
-		else {
-			double timeReduce = 0;
-			if (entityList[executor].timeChronoLeft > 0) {
-				timeReduce = min(10, assetList[action].duration / 3);
-			}
-			timeToComplete = assetList[action].duration + time - timeReduce;
-		}
-		return Entity(action, timeToComplete, time, 0, 0, executor, target, complete);
-	}
-	void step(int action, int executor, int chronoTarget) {
-		double timeToAction = computeTimeToAction(action, executor, chronoTarget);
-		double slidingTime = timeToAction;
-		//need to iterate through all incomplete items
-		while (true) {
-			int idx = getNextComplete();
-			if (idx == -1) break;
-			double timeComplete = entityList[idx].timeToComplete - time;
-			if (timeComplete <= slidingTime) {
-				elapseTime(timeComplete);
-				completeEvent(idx);
-				slidingTime = computeTimeToAction(action, executor, chronoTarget);
-			}
-			else break;
-		}
-		elapseTime(slidingTime);
-		completeAction(action, executor, chronoTarget);
-		gymReturn.reward = -timeToAction;
-		gymReturn.done = isDone();
-		gymReturn.state = state;
-		reward = -timeToAction;
-		done = isDone();
-	}
-
-	double getTimeNextSupplyDone() {
-		double minTime = 10000;
-		for (int i = 0; i < inProgressCount; i++) {
-			const Entity& e = entityList[inProgressEntities[i]];
-			if (!e.isComplete && assetList[e.assetType].supProvide > 0) {
-				minTime = min(e.timeToComplete, minTime);
-			}
-		}
-		return minTime - time;
-	}
-
-	double computeTimeToAction(int action, int executor, int chronoTarget) {
-		if (action == n.chronoboost) {
-			double energyAvailableAt;
-			if (entityList[executor].isComplete) {
-				energyAvailableAt = max(0.0, 50 - entityList[executor].energy) / REGEN_RATE;
-			}
-			else {
-				energyAvailableAt = entityList[executor].timeToComplete - time;
-			}
-			return max(entityList[chronoTarget].timeChronoLeft, energyAvailableAt);
-		}
-
-		double timeMinerals;
-		double timeGas;
-		double timeSupply;
-		double timeQueue;
-		const AssetStats& a = assetList[action];
-		double mineralRate = mRate();
-		double gasRate = gRate();
-
-		//also need to consider when executor has a free queue
-
-		if (mineralRate == 0) timeMinerals = 0;
-		else {
-			timeMinerals = (a.minCost - minerals) / mineralRate;
-		}
-		if (gasRate == 0) timeGas = 0;
-		else {
-			timeGas = (a.gasCost - gas) / gasRate;
-		}
-		if (maxSupply >= supply + a.supCost) timeSupply = 0;
-		else {
-			timeSupply = getTimeNextSupplyDone();
-		}
-		if (executor > 0) {
-			timeQueue = entityList[executor].timeQueueFree - time;
-		}
-		else {
-			timeQueue = 0;
-			
-		}
-		double largest = timeMinerals;
-		if (timeGas > largest) largest = timeGas;
-		if (timeSupply > largest) largest = timeSupply;
-		if (timeQueue > largest) largest = timeQueue;
-		return largest;
-	}
-
-	double mRate() {
-		return mProbeCount;
-	}
-	double gRate() {
-		return gProbeCount;
-	}
-
-	void elapseTime(double timeElapse) {
-		minerals += timeElapse * mRate();
-		gas += timeElapse * gRate();
-		for (Entity& e : entityList) {
-			//e.timeChronoLeft = max(0.0, e.timeChronoLeft - timeElapse);
-			//e.timeQueueFree = max(0.0, e.timeQueueFree - timeElapse);
-			//e.timeToComplete = max(0.0, e.timeToComplete - timeElapse);
-			e.energy = min(200.0, e.energy + timeElapse * REGEN_RATE);
-		}
-		time += timeElapse;
-	}
-
-	void removeInProgress(int idx) {
-		if (inProgressCount == 1) {
-			inProgressCount = 0;
-			return;
-		}
-
-		for (int i = 0; i < inProgressCount; i++) {
-			if (inProgressEntities[i] == idx) {
-				inProgressEntities[i] = inProgressEntities[inProgressCount - 1];
-				inProgressCount--;
-				return;
-			}
-		}
-	}
-
-	void completeEvent(int idx) {
-		int aType = entityList[idx].assetType;
-		if (aType == n.nexus) {
-			entityList[idx].energy = 50;
-		}
-		else if (aType == n.probe) {
-			mProbeCount++;
-		}
-		else if (aType == n.assimilator) {
-			int probesChange = max(0, min(3, mProbeCount - 3));
-			mProbeCount -= probesChange;
-			gProbeCount += probesChange;
-		}
-		maxSupply += assetList[entityList[idx].assetType].supProvide;
-		entityList[idx].isComplete = true;
-		removeInProgress(idx);
-	}
-
-	void completeAction(int action, int executor, int target) {
-		if (action == n.chronoboost) {
-			entityList[executor].energy -= 50;
-			entityList[target].timeChronoLeft = 20;
-			double timeStillQueued = max(0.0, entityList[target].timeQueueFree - time);
-			double timeShift = min(10.0, timeStillQueued / 3);
-			entityList[target].timeQueueFree -= timeShift;
-			entityList[entityList[target].child].timeToComplete -= timeShift;
-			return;
-		}
-		const AssetStats& a = assetList[action];
-		maxFutureSupply += a.supProvide;
-		minerals -= a.minCost;
-		gas -= a.gasCost;
-		supply += a.supCost;
-		entityList.push_back(loadEntity(action, executor, target, false));
-		if (executor > 0) {
-			entityList[executor].timeQueueFree = a.duration + time;
-			entityList[executor].child = entityList.size() - 1;
-		}
-		inProgressEntities[inProgressCount++] = entityList.size() - 1;
-		state[action] += 1;
-	}
-
-	int getNextComplete() {
-		double minCompleteTime = 10000;
-		int nextComplete = -1;
-		for (size_t i = 0; i < inProgressCount; i++) {
-			const Entity& e = entityList[inProgressEntities[i]];
-			if (!e.isComplete && e.timeToComplete < minCompleteTime) {
-				minCompleteTime = e.timeToComplete;
-				nextComplete = inProgressEntities[i];
-			}
-		}
-		return nextComplete;
-	}
-
-	bool isDone() {
-		//check if entity counter >= goalstate
-		for (size_t i = 0; i < state.size(); i++) {
-			if (state[i] < goalState[i]) {
-				return false;
-			}
-		}
-		return true;
-	}
-
-	bool techExists(int i) {
-		for (int req : assetList[i].techReqs) {
-			if (state[req] == 0) {
-				return false;
-			}
-		}
-		return true;
-	}
-
-	bool maxAssimilators(const AssetStats& a) {
-		return (a.name == n.assimilator && state[n.assimilator] > state[n.nexus] * 2);
-	}
-
-	vector<int > getViableActions() {
-		//needs to pass tech, supply, resource, maxCount checks
-		//vector<int > viableActions;
-		vector<int > viableActions(100);
-		int viableCount = 0;
-		for (const AssetStats& a : assetList) {
-			if (maxAssimilators(a)) continue;
-			bool techExists = true;
-			for (int req : a.techReqs) {
-				if (state[req] == 0) {
-					techExists = false; break;
-				}
-			}
-			bool isViable = (
-				techExists
-				&& (a.supCost + supply <= maxFutureSupply)
-				&& (mProbeCount > 0 || a.minCost == 0)
-				&& (gProbeCount > 0 || a.gasCost == 0)
-				&& (state[a.name] < a.maxQuantity)
-				);
-			if (isViable) {
-				viableActions[viableCount++] = a.name;
-			}
-		}
-		//chronoboost
-		if (state[n.nexus] > 0) {
-			viableActions[viableCount++] = n.chronoboost;
-		}
-		viableActions.resize(viableCount);
-		return viableActions;
-	}
-	vector<int> getViableExecutors(int action) {
-		int executorName;
-		if (action == n.chronoboost) {
-			executorName = n.nexus;
-		}
-		else { executorName = assetList[action].parent; }
-		vector<int> viableExecutors;
-		for (int i = 0; i < entityList.size(); i++) {
-			if (entityList[i].assetType == executorName) {
-				viableExecutors.push_back(i);
-			}
-		}
-		return viableExecutors;
-	}
-
-	vector<int> getViableTargets(int action, int executor) {
-		vector<int> viableTargets;
-		//only chronoboost has targets
-		if (action != n.chronoboost) return viableTargets;
-		for (int i = 0; i < entityList.size(); i++) {
-			if (assetList[entityList[i].assetType].canChrono) {
-				viableTargets.push_back(i);
-			}
-		}
-		return viableTargets;
-	}
-
-	vector<vector<double>> getState() {
-		int maxEntities = 100;
-		vector<vector<double>> returnState(maxEntities, vector<double>(6, 0));
-		for (int i = 0; i < entityList.size(); i++) {
-			if (i == maxEntities)break;
-			returnState[i][0] = entityList[i].assetType;
-			returnState[i][1] = max(0.0, entityList[i].timeToComplete - time);
-			returnState[i][2] = max(0.0, entityList[i].timeQueueFree - time);
-			returnState[i][3] = max(0.0, entityList[i].timeChronoLeft - time);
-			returnState[i][4] = entityList[i].energy;
-			returnState[i][5] = (double)entityList[i].isComplete;
-		}
-		return returnState;
-	}
-
-	int pickNextExecutor(vector<int> viableExecutors) {
-		if (viableExecutors.size() == 0) return NULL;
-		double timeFree = 10000;
-		int nextExecutor = 0;
-		for (int x : viableExecutors) {
-			if (entityList[x].timeQueueFree < timeFree) {
-				timeFree = entityList[x].timeQueueFree;
-				nextExecutor = x;
-			}
-		}
-		return nextExecutor;
 	}
 };
 
@@ -683,6 +715,12 @@ PYBIND11_MODULE(probotest1, m) {
 
     )pbdoc";
 
+	pybind11::class_<GeneralLoader>(m, "GeneralLoader")
+		.def(pybind11::init<>())
+		.def("createFreshEnv", &GeneralLoader::createFreshEnv)
+		.def("deepCopyEnv", &GeneralLoader::deepCopyEnv)
+		;
+
     pybind11::class_<ProtossMacroEnv>(m, "ProtossMacroEnv")
         .def(pybind11::init<>())
         .def("step", &ProtossMacroEnv::step)
@@ -690,14 +728,12 @@ PYBIND11_MODULE(probotest1, m) {
 		.def("getEntityList", &ProtossMacroEnv::getState)
 		.def("getViableActions", &ProtossMacroEnv::getViableActions)
 		.def("getViableExecutors", &ProtossMacroEnv::getViableExecutors)
-		.def("getViableTargets", &ProtossMacroEnv::getState)
 		.def("pickNextExecutor", &ProtossMacroEnv::pickNextExecutor)
         .def_readwrite("minerals", &ProtossMacroEnv::minerals)
 		.def_readwrite("gas", &ProtossMacroEnv::gas)
 		.def_readwrite("time", &ProtossMacroEnv::time)
 		.def_readwrite("supply", &ProtossMacroEnv::supply)
 		.def_readwrite("supplycap", &ProtossMacroEnv::maxSupply)
-		.def_readwrite("reward", &ProtossMacroEnv::minerals)
 		.def_readwrite("probeCountOnMinerals", &ProtossMacroEnv::mProbeCount)
 		.def_readwrite("probeCountOnGas", &ProtossMacroEnv::gProbeCount)
 		.def_readwrite("state", &ProtossMacroEnv::state)
